package garcia.alex.talia;

//imports
import garcia.alex.talia.entity.mob.Player;
import garcia.alex.talia.graphics.Screen;
import garcia.alex.talia.input.Keyboard;
import garcia.alex.talia.input.Mouse;
import garcia.alex.talia.level.Level;
import garcia.alex.talia.level.TileCoordinate;

import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferStrategy;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferInt;

import javax.swing.JFrame;

//Game class
public class Game extends Canvas implements Runnable {
	private static final long serialVersionUID = 1L;//java convention
	
		/*
		 * These are all of the declared and/or initialized variables
		 */
	//window size
	private static int width = 300;
	private static int height = (width / 16 * 9);
	private static int scale = 3; //size the game renders  is 1 size of window is this variable
	//thread
	private Thread thread;
	//jframe
	private JFrame frame;
	//game loop state
	private Keyboard key;
	private Level level;
	private Player player;
	private boolean running = false;
	public static String title = "Talia";
	private Screen screen;
	private BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB); //an image with a buffer
	private int[] pixels = ((DataBufferInt)image.getRaster().getDataBuffer()).getData();//accesses the created image
	
		/*
		 * These are all of the methods
		 */
	
	public Game() {
		Dimension size = new Dimension(width * scale, height * scale);
		setPreferredSize(size);
		screen = new Screen(width, height);
		frame = new JFrame();
		key = new Keyboard();
		level = Level.spawn;
		TileCoordinate playerSpawn = new TileCoordinate (20, 54);
		player = new Player(playerSpawn.x(), playerSpawn.y(), key);
		player.init(level);
		addKeyListener(key);
		
		Mouse mouse = new Mouse();
		addMouseListener(mouse);
		addMouseMotionListener(mouse);
	}//end Game()
	
	public static int getWindowWidth(){
		return width * scale;
	}
	
	public static int getWindowHeight(){
		return height * scale;
	}
	
	//thread start
	public synchronized void start() {
		running = true; //begins the gameloop
		thread = new Thread (this, "Display");
		thread.start();
	}//end start()
	
	//to stop a thread
	public synchronized void stop() {
		running = false; //ends the gameloop
		try{
			thread.join();
		} catch (InterruptedException e){
			e.printStackTrace();
		}//end catch
	}//end stop()
	
	public void run() { 
		long lastTime = System.nanoTime();
		long timer = System.currentTimeMillis();
		final double ns = 1000000000.0 / 60.0;//will update the logic 60x a second
		double delta = 0;
		int frames = 0;
		int updates = 0;
		requestFocus();//makes the screen ready for input
		while(running) { //the gameloop
			long now = System.nanoTime();
			delta += (now - lastTime) / ns;
			lastTime = now;
			while (delta >= 1){
				update();//updates the game, logic
				updates++;
				delta--;
			}//end while loop
			render();//renders the game, graphics
			frames ++;
			if (System.currentTimeMillis() - timer > 1000){//this if statement calculated the fps
				timer += 1000;
				System.out.println(updates + "ups, " + frames + "fps;");
				frame.setTitle(title + "  |  " + frames + "fps");//shows fps on the window
				updates = 0;
				frames = 0;
			}//end fps counter if statement
		}//end gameloop
	}//end run()
	
	public void update() { //updates the game, logic
		key.update();
		player.update();
		level.update();
	}//end update()
	
	public void render(){ //renders the game, graphics
		BufferStrategy bs = getBufferStrategy();
		if (bs == null) {
			createBufferStrategy(3);//creates three buffers
			return;
		}//end bufferstrategy if statement
		screen.clear();//clears the screen
		int xScroll = player.x - screen.width / 2;
		int yScroll = player.y - screen.height / 2;
		level.render(xScroll, yScroll, screen);		
		player.render(screen);
				
		for (int i = 0; i < pixels.length; i++) {
			pixels[i] = screen.pixels[i];
		}//end pixel render for loop
		Graphics g = bs.getDrawGraphics();//links graphics and the buffer
		/*
		 * All graphics are displayed between  the above statement and g.dispose
		 */
		g.drawImage(image, 0, 0, getWidth(), getHeight(), null);
		//g.fillRect(Mouse.getX(), Mouse.getY(), 64, 64);
		g.dispose();//removes the graphics
		bs.show();//makes the next avalible buffer visible	
	}//end render()
	
	//main method
	public static void main(String[] args){
		Game game = new Game();
		game.frame.setResizable(false);//doesnt allow window to be resized
		game.frame.setTitle(title);//set title of window
		game.frame.add(game);//fills the window with the game object
		game.frame.pack();//frame is same size as window
		game.frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);//closes application when clicking the red x in the corner
		game.frame.setLocationRelativeTo(null);//centers the window
		game.frame.setVisible(true);//shows the window
		game.start();//starts the game
	}//end main
}//end Game class
