package garcia.alex.talia.graphics;

import garcia.alex.talia.level.tile.Tile;
import java.util.Random;

public class Screen {
	
		/*
		 * These are all of the declared and/or initialized variables
		 */
	public int width;
	public int height;
	public int[] pixels;
	public final int MAP_SIZE = 64;
	public final int MAP_SIZE_MASK = MAP_SIZE - 1;
	public int xOffset, yOffset;
	public int[] tiles = new int [64 * 64];
	private Random random = new Random();
	
		/*
		 * These are all of the methods
		 */
	public Screen(int width, int height){
		this.width = width;
		this.height = height;
		pixels = new int[width * height];
		for (int i = 0; i < MAP_SIZE * MAP_SIZE; i++){
			tiles[i] = random.nextInt(0xffffff);
		}//end for loop
	}//end screen()
	
	public void clear() {
		for (int i = 0; i < pixels.length; i++){
			pixels[i] = 0;
		}//end clear pixels for loop
	}//end clear()
	
	public void renderTile(int xp, int yp, Tile tile){
		xp -= xOffset;
		yp -= yOffset;
		for (int y = 0; y < tile.sprite.SIZE; y++){
			int ya = y + yp;//ya is y absolute
			for (int x = 0; x < tile.sprite.SIZE; x++){
				int xa = x + xp;//xa is x absolute
				if (xa < -tile.sprite.SIZE || xa >= width || ya < 0 || ya >= height) break; //only renders what is on the screen
				if (xa < 0) xa = 0;
				pixels[xa + ya * width] = tile.sprite.pixels[x + y * tile.sprite.SIZE]; //where the sprite gets rendered and what pixels of the sprite get rendered
			}//end x for loop
		}//end y for loop
	}//end rendertile
	
	public void renderSprite(int xp, int yp, Sprite sprite, boolean fixed){
		if (fixed){
			xp -= xOffset;
			yp -= yOffset;
		}
		for (int y = 0; y < sprite.getHeight(); y++){
			int ya = y + yp;//ya is y absolute
			for (int x = 0; x < sprite.getWidth(); x++){
				int xa = x + xp;//xa is x absolute
				if (xa < 0 || xa >= width || ya < 0 || ya >= height) break; //only renders what is on the screen
				int col = sprite.pixels[x + y * sprite.SIZE]; //where the sprite gets rendered and what pixels of the sprite get rendered, (this line and next), will not render pink pixels
				if (col != 0xffff00ff)pixels[xa + ya * width] = col; 
			}//end x for loop
		}//end y for loop
	}//end rendersprite()
	
	public void renderPlayer(int xp, int yp, Sprite sprite){
		xp -= xOffset;
		yp -= yOffset;
		for (int y = 0; y < 16; y++){
			int ya = y + yp;//ya is y absolute
			for (int x = 0; x < 16; x++){
				int xa = x + xp;//xa is x absolute
				if (xa < -16 || xa >= width || ya < 0 || ya >= height) break; //only renders what is on the screen
				if (xa < 0) xa = 0;
				int col = sprite.pixels[x + y * 16];				 //will not render pink pixels
				if (col != 0xffff00ff)pixels[xa + ya * width] = col; //will not render pink pixels
			}//end x for loop
		}//end y for loop
	}
	
	//this method is to be used if you want to flip a sprite at all
	public void renderPlayer(int xp, int yp, Sprite sprite, boolean xFlip, boolean yFlip){
		xp -= xOffset;
		yp -= yOffset;
		int xf = 0, yf = 0;
		
		for (int y = 0; y < 32; y++){
			int ya = y + yp;//ya is y absolute
			if (yFlip == true) yf = 31 - y;
			else yf = y; 
			for (int x = 0; x < 32; x++){
				int xa = x + xp;//xa is x absolute
				if (xFlip == true) xf = 31 - x;
				else xf = x; 
				if (xa < -32 || xa >= width || ya < 0 || ya >= height) break; //only renders what is on the screen
				if (xa < 0) xa = 0;
				int col = sprite.pixels[xf + yf * 32];
				if (col != 0xffff00ff)pixels[xa + ya * width] = col; 
			}//end x for loop
		}//end y for loop
	}
	
	public void setOffset(int xOffset, int yOffset){
		this.xOffset = xOffset;
		this.yOffset = yOffset;
	}

}//end screen class
