package garcia.alex.talia.level;

import garcia.alex.talia.entity.Entity;
import garcia.alex.talia.entity.particle.Particle;
import garcia.alex.talia.entity.projectile.Projectile;
import garcia.alex.talia.graphics.Screen;
import garcia.alex.talia.level.tile.Tile;

import java.util.ArrayList;
import java.util.List;

public class Level {
		/*
		 * These are all of the declared and/or initialized variables
		 */
	protected int width, height;
	protected int[] tiles;//contains loaded level's tiles
	public static Level spawn = new SpawnLevel("/levels/spawn.png");
	
	private List<Entity> entities = new ArrayList<Entity>();
	private List<Projectile> projectiles = new ArrayList<Projectile>();
	private List<Particle> particles = new ArrayList<Particle>();
	
		/*
		 * These are all of the methods
		 */
	public Level(int width, int height){//random level generation
		this.width = width;
		this.height = height;
		tiles = new int[width * height];
		generateLevel();
	}//end level (generated)

	public Level(String path){//loading a saved level
		loadLevel(path);
		generateLevel();
	}//end level (loaded)
	
	protected void generateLevel(){
	}//end generate level
	
	protected void loadLevel(String path){
	}//end loadlevel
		
	public void update(){
		remove();
		for (int i = 0; i < entities.size(); i++){//updates entities
			entities.get(i).update();
		}
		for (int i = 0; i < projectiles.size(); i++){//updates projectiles
			projectiles.get(i).update();
		}
		for (int i = 0; i < particles.size(); i++){//updates particles
			particles.get(i).update();
		}
	}//end update
	
	private void remove(){
		for (int i = 0; i < entities.size(); i++){//removes entities
			if(entities.get(i).isRemoved()) entities.remove(i);
		}
		for (int i = 0; i < projectiles.size(); i++){//removes projectiles
			if(projectiles.get(i).isRemoved()) projectiles.remove(i);
		}
		for (int i = 0; i < particles.size(); i++){//removes particles
			if(particles.get(i).isRemoved()) particles.remove(i);
		}
	}
	
	public List<Projectile> getProjectiles(){
		return projectiles;
	}
	
	//	private void time(){ //unused as of now, commenting it out
	//	}//end time
	
	public boolean tileCollision(int x, int y, int size, int xOffset, int yOffset) {//x and y are entity position, nx and ny are position entity is heading to
		boolean solid = false;
		for( int c = 0; c < 4; c++){
			int xt = (x - c % 2 * size + xOffset) >> 4;//deals with collision box of projectile
			int yt = (y - c / 2 * size + yOffset) >> 4;
			if(getTile( xt, yt).solid()) solid = true;
		}
		return solid;
	}
	
	public void render(int xScroll, int yScroll, Screen screen){
		screen.setOffset(xScroll, yScroll);
		int x0 = xScroll >> 4;//these variables define the screen render zone (pins)
		int x1 = (xScroll + screen.width + 16) >> 4;
		int y0 = yScroll >> 4;
		int y1 = (yScroll + screen.height + 16) >> 4;

		for (int y = y0; y < y1; y++){//renders level
			for (int x = x0; x < x1; x++){
				getTile(x, y).render(x, y, screen);	
			}
		}
		
		for (int i = 0; i < entities.size(); i++){//renders entities
			entities.get(i).render(screen);
		}
		for (int i = 0; i < projectiles.size(); i++){//renders projectiles
			projectiles.get(i).render(screen);
		}
		for (int i = 0; i < particles.size(); i++){//renders particles
			particles.get(i).render(screen);
		}
	}//end render
	
	public void add(Entity e){
		e.init(this);
		if (e instanceof Particle){
			particles.add((Particle)e);
		}else if(e instanceof Projectile){
			projectiles.add((Projectile)e);			
		}else{
			entities.add(e);
		}//end if else statements
	}//end add entity
	
	
	//plain = 0xff00ff08
	//grass = 0xff007f02
	//flower = 0xffff4828
	//rock = 0xff465054
	//hedge = 0xff00bf03
	//bricks = 0xff9aafb7
	//flooring = 0xffc67c21
	public Tile getTile(int x, int y){
		
		int rInt = 0;
		
		if (x < 0 || y < 0 || x >= width || y >= height) return Tile.voidTile;//solves out of bound crash
		if (tiles[x + y * width] == Tile.col_plain1){
			if (rInt == 0)return Tile.plain1;
			if (rInt == 1)return Tile.plain2;
		}
		if (tiles[x + y * width] == Tile.col_grass1){
			if (rInt == 0)return Tile.grass1;
			if (rInt == 1)return Tile.grass2;
		}
		if (tiles[x + y * width] == Tile.col_redFlower1){
			if (rInt == 0)return Tile.redFlower1;
			if (rInt == 1)return Tile.pinkFlower1;
		}
		if (tiles[x + y * width] == Tile.col_rock1){
			if (rInt == 0)return Tile.rock1;
			if (rInt == 1)return Tile.rock2;
		}
		if (tiles[x + y * width] == Tile.col_hedge1){
			if (rInt == 0)return Tile.hedge1;
			if (rInt == 1)return Tile.hedge2;
		}
		if (tiles[x + y * width] == Tile.col_bricks1){
			if (rInt == 0)return Tile.bricks1;
			if (rInt == 1)return Tile.bricks2;
		}
		if (tiles[x + y * width] == Tile.col_flooring1){
			if (rInt == 0)return Tile.flooring1;
			if (rInt == 1)return Tile.flooring2;
		}
		return Tile.voidTile;
	}
	
}//end level class
